/* V 31.05.2013
     Afficheur LCD-jumbo "récupération CANT COURBON 1993"
*/
// definition des entrées sorties pour un ARDUINO MEGA
#define lcdj_0 32	// poids faible
#define lcdj_1 33
#define lcdj_2 34
#define lcdj_3 35	// poids fort
#define lcdj_WE 36	// commande d'écriture
#define lcdj_CS1 37  // non utilisé si un seul afficheur (tirer à la masse)
//#define lcdj_CS2 38  // non utilisé si un seul afficheur
//#define lcdj_CS3 39  // non utilisé si un seul afficheur
//#define lcdj_RE 40  // non utilisé (tirer au +5 Volt)
//#define lcdj_READY 41  // non utilisé (laisser en l'air)

/*
// definition des entrées sorties pour un ARDUINO UNO
#define lcdj_0 2
#define lcdj_1 3
#define lcdj_2 4
#define lcdj_3 5
#define lcdj_WE 6
#define lcdj_CS1 7  // non utilisé si un seul afficheur (tirer à la masse)
#define lcdj_CS2 8  // non utilisé si un seul afficheur
#define lcdj_CS3 9  // non utilisé si un seul afficheur
//#define lcdj_RE 10  // non utilisé (tirer au +5 Volt)
//#define lcdj_READY 11  // non utilisé (laisser en l'air)
*/

// tempo en microsecondes entre commandes lcd
#define lcdj_delay 50  // 14 au moins, 50 est plus sûr ! 
// la notice indique 2,5 à 3,5 microsec., mais il faut au moins 14 ...

/*
Assez bizarrement, les 8 segments d'un digit ne sont pas contigus dans la mémoire. 
Cela vient d'une simplification hazardeuse de la conception du circuit imprimé entre le contrôleur et le verre LCD.
Les 3 segments inférieurs et le point de 2 digits contigus sont dans le même octet (0,1,2)
Les 4 segments supérieurs sont sur dautres ovtets (6,5,4)
Par ailleurs, des : existent sur le verre entre les digits 1 et 2 et aussi 3 et 4, mais ne sont pas câblés, 
bien que le contrôleur puisse piloter 8 digits complets .
D'où le système donnant le numero de bit dans la RAM correspondant à chaque segment de chaque digit 
pour a,b,c,d,e,f,g,p(DP) iseg=0 à 7, et la formule est :
ibit_ram = lcdj_segcst[iseg].seg0 + lcdj_segcst[iseg].seg1 * idigit;
*/
struct lcdjsegcst { int seg0, seg1; } ;
struct lcdjsegcst lcdj_segcst[] = {
  	{45, -4},  //a
  	{44, -4},  //b
  	{2, 4},    //c
  	{1, 4},    //d
  	{0, 4},    //e
  	{46, -4},  //f
  	{47, -4},  //g
  	{3, 4}     //p (DP)
};

// deux-points séparateurs
#define lcdj_colon1 48
#define lcdj_colon2 49

// Ensuite, il nous faut une table donnant les segments à allumer en fonction du caractère à afficher :
struct lcdjseg { char c; byte seg; } ;
struct lcdjseg lcdj_seg[] = {
//       'X', Bpgfedcba    CHIFFRES
  	{'0', B00111111},
	{'1', B00000110},
	{'2', B01011011},
	{'3', B01001111},
	{'4', B01100110},
	{'5', B01101101},
	{'6', B01111101},
	{'7', B00000111},
	{'8', B01111111},      
	{'9', B01101111},  
//       'X', Bpgfedcba    LETTRES (les minuscules sont automatiquement affichées comme les majuscules)
	{'A', B01110111},  // A 
	{'B', B01111100},  // b                  schema et noms des segments
	{'C', B00111001},  // C    ***a***
	{'D', B01011110},  // d    *     *
	{'E', B01111001},  // E    f     b
	{'F', B01110001},  // F    ***g***
	{'G', B00111101},  // g    *     *
	{'H', B01110110},  // h    e     c
	{'I', B00000100},  // i    ***d***  p (point decimal)
	{'J', B00011110},  // J
	{'K', B01110000},  // K (pas génial)
	{'L', B00111000},  // L 
	{'M', B00110111},  // M
	{'N', B01010100},  // n
  	{'O', B01011100},  // o
  	{'P', B01110011},  // P
  	{'Q', B01100111},  // q
  	{'R', B01010000},  // r
	{'S', B01101101},  // S
	{'T', B01111000},  // t
	{'U', B00011100},  // u
	{'V', B00111110},  // V
	{'W', B01111110},  // W
	{'X', B01010010},  // X (pas très génial !)
	{'Y', B01100110},  // Y
	{'Z', B01011011},  // Z idem au 5
//       'X', Bpgfedcba      SIGNES DIVERS
	{'?', B01010011},
	{'+', B00000001},    // tiret du haut
	{'-', B01000000},    // tiret du milieu
	{'_', B00001000},    // tiret du bas
	{'=', B01001000},
	{'°', B01100011},  // degrés
	{'.', B10000000},  // point seul (sinon gestion particulière)
	{' ', B00000000}   // dernier caractère pris par défaut si pas trouvé !
     } ;
//       'X', Bpgfedcba
//	{'A', B01011111},  // a 
//	{'C', B01011000},  // c    
//	{'E', B01111011},  // e    
// nombre de caractères définis
#define lcdj_seg_nb  (sizeof(lcdj_seg)/sizeof(lcdj_seg[0]))

// table des bits de RAM du lcd jumbo (6 digits de 8 bits + 2 deux-points XY)
boolean lcdj_ram[56]; //,lcdj_ramv[50];  // boolean values
// edcpbafgedcpbafgedcpbafgedcpbafgedcpbafgedcpbafgXY
// 01234567890123456789012345678901234567890123456789012345

// buffer des caractères à afficher
// ce buffer comporte : 6 caractères, 6 points, 2 deux-points, une fin (un null pour éviter les ennuis lors d'envoi sur Serial!)
char lcdj_buf[15],lcdj_bufv[15];
// ABCDEF......::
// 01234567890123

//--------------------------------********************--------------------------------
// positionne les bits nécessaires pour un digit
void lcdj_affiche_digit (byte car, int digit) {
byte segments;
int ibit,inti,i,j,k,etat;

  j=0;
// conversion minuscules -> majuscules
  segments=car; if (car >= 'a' && car <= 'z') segments = (car & 0x9F) | 0x40 ;
// recherche caractère à afficher
  while ((segments != lcdj_seg[j].c) && (j < lcdj_seg_nb-1 )) { j++; }
// etats des segments pour ce caractère
  segments = lcdj_seg[j].seg;

// transfert des état de segments dans le buffer RAM (sans le point decimal, traités plus loin)
  for (i=0;i<7;i++) {
    ibit = lcdj_segcst[i].seg0 + lcdj_segcst[i].seg1*digit;
// masque à 1 bit positionné sur le bit courant
    j=1<<i;
// mise a jour table RAM lcdj
    if ((segments & j) != 0) {lcdj_ram[ibit] = HIGH;} else {lcdj_ram[ibit] = LOW;}
/*
// envoi commande bit à bit
    etat=0x4000; if ((segments & j) != 0) etat=0x6000;
    etat = etat | ibit; lcdj_quartet(etat);
*/
  }
}
//--------------------------------********************--------------------------------
// envoi des 4 octets de "value" au lcd JUMBO 
void lcdj_quartet(int value) {
// value = 4 octets à traiter
int etat,val,i;
// pour les 4 octets
  for (i=3;i>=0;i--) {
// validation du chip HITACHI concerné
    digitalWrite(lcdj_CS1, LOW);    // validation chip
// décalage à droite selon quartet à envoyer
    val=value; 
    val = val >> 4*i;
// horodatage_serial(); Serial << "quartet=" << _HEX(val) << endl;
    etat = val & 1; digitalWrite(lcdj_0, etat);
    val = val >> 1; etat = val & 1;  digitalWrite(lcdj_1, etat);
    val = val >> 1; etat = val & 1;  digitalWrite(lcdj_2, etat);
    val = val >> 1; etat = val & 1;  digitalWrite(lcdj_3, etat);

    delayMicroseconds(lcdj_delay);    digitalWrite(lcdj_WE, LOW);    // impulsion pour écrire
    delayMicroseconds(lcdj_delay);    digitalWrite(lcdj_WE, HIGH);    // fin impulsion

    delayMicroseconds(lcdj_delay);    digitalWrite(lcdj_CS1, HIGH);    // dé-validation chip
  }
return;
}
//--------------------------------********************--------------------------------
void lcdj_envoi_ram() {
  byte octet;
  int digit,i,j,k;
 
// pour eviter les crashes ...
   lcdj_buf[sizeof(lcdj_buf)-1] = '\0';

// traitement des 5 points 'xxxxxx.....!xx'
  for (i=6;i<11;i++) {
    if (lcdj_bufv[i] != lcdj_buf[i]) {
      if (lcdj_buf[i] > ' ') {
        lcdj_ram[(i-6)*4+3]=1;
      } else {
        lcdj_ram[(i-6)*4+3]=0;
      }
    }
  }
  
// traitement des : 'aaaaaaxxxxxx::'
  j=0;
  for (i=12;i<14;i++) {
    if (lcdj_bufv[i] != lcdj_buf[i]) {
        j=6;    // modifié
//Serial << ":: BOUGE i=" << i << "<" << lcdj_bufv << ">>" << lcdj_buf << ">" << endl;
	if (lcdj_buf[i] > ' ') {
          lcdj_ram[lcdj_colon1+i-12]=1; // allumage
//	  k=0x6000;	// mise à 1
	} else {
	  lcdj_ram[lcdj_colon1+i-12]=0;  // extinction
//	  k=0x4000;	// mise à 0
	}
// envoi par commande bit
//        k = k | (lcdj_colon1+i-12); lcdj_quartet(k);
    }
  }
// envoi de l'octet entier (qui ne contient que les 2 ::) si un des 2 a changé
    if (j>0) {
         octet = (lcdj_ram[lcdj_colon2] << 7) | (lcdj_ram[lcdj_colon1] << 6);
//Serial << "envoi:: j=" << j << " " << _HEX(octet) << endl;
         lcdj_quartet(0x0100*j + octet);
    }

// traitement des 6 digits 2 à 2
    for (digit=0;digit<6;digit+=2) {
// si caractère ou points ont changés
//Serial << "digit=" << digit << "<" << lcdj_bufv << ">>" << lcdj_buf << ">" << endl;
      if ((lcdj_bufv[digit] != lcdj_buf[digit]) or (lcdj_bufv[digit+1] != lcdj_buf[digit+1]) or
          (lcdj_bufv[digit+6] != lcdj_buf[digit+6]) or (lcdj_bufv[digit+7] != lcdj_buf[digit+7])) {
//Serial << "digit CHANGE" << endl;
// envoi par octets a partir de la table de bits
         j=digit/2;
// paquets de segments du bas
         k=j*8; octet=0; for (i=k;i<k+8;i++) { octet = octet << 1; octet = octet | lcdj_ram[i]; }
         lcdj_quartet(0x0100*j + octet);
// paquets de segments du haut        
         k=(5-j)*8; octet=0; for (i=k;i<k+8;i++) { octet = octet << 1; octet = octet | lcdj_ram[i]; }
         lcdj_quartet(0x0100*(5-j) + octet);
	   }
    }
}
//--------------------------------********************--------------------------------
// sous-programme qui surveille si on doit afficher quelquechose
// appeler ce module dans la boucle principale, toutes les 100ms est suffisant
void lcdj_affiche() {
int i;
      if (strcmp(lcdj_bufv,lcdj_buf) !=0) {
//Serial << "grafcet_lcdj=" << grafcet_lcdj << " " << lcdj_buf << endl;
        for (i=0;i<6;i++) {
          lcdj_affiche_digit (lcdj_buf[i],i);
//Serial << "i=" << i << " " << lcdj_buf[i] << endl;
        }
        lcdj_envoi_ram();
// mise en mémoire
        strcpy(lcdj_bufv,lcdj_buf);
      }
}
//--------------------------------********************--------------------------------
// initialisation du LCD JUMBO
void lcdj_init() {
int i;
// positionnement sorties
    digitalWrite(lcdj_WE, HIGH);
    digitalWrite(lcdj_CS1, HIGH);

// reset compteur de quartets
    i=0xCCCC; lcdj_quartet(i);
// mode setting (deux fois , selon notice)
    i=0x8400; lcdj_quartet(i);
    i=0x8400; lcdj_quartet(i);
  
// init buffers
//     memset (lcdj_buf,'\0',sizeof(lcdj_buf));
     memset (lcdj_bufv,'\0',sizeof(lcdj_bufv));

// TEST de démarrage
// tout allumer
     memset (lcdj_buf,'8',sizeof(lcdj_buf)-1);
// tout éteindre
//     memset (lcdj_buf,' ',sizeof(lcdj_buf));
}
//--------------------------------********************--------------------------------


